/* =============================================================================
 UPC - EETAC - CSD - http://digsys.upc.edu 
  
 * Integrating MPLAB X + XC8 + MCC + Proteus VSM 
 
 * Explorer 8 board user guide: 
        https://ww1.microchip.com/downloads/en/DeviceDoc/40001812B.pdf   

 * Materials adapted from code Explorer 8 tutorials (Explorer8_Labs_v2.zip) at:
        https://www.microchip.com/en-us/development-tool/DM160228
 
 * First sample project. Hello World using functions from Explorer 8 board
 
 * Microcontroller PIC18F46k22 
 * https://www.microchip.com/en-us/product/PIC18F46K22 
============================================================================= */

/* 	============================================================================
    Board buttons S1 and S2.            LEDs RB3 RB2
    
    S2 and S1 read in the main loop
    Explorer LCD display used by default to represent ASCII messages
   	========================================================================= */

/* 	============================================================================
 IMPORTANT NOTE: This code is used with MCC (MPLAB code configurator) 
 * Do not update or force update main.c and main.h files because 
 * Be aware of what to update for other c and h files generated by MCC
   	========================================================================= */

/* 	============================================================================
 Use this project as a template for copying and adapting new P9 CSD applications 
 requiring Explorer 8 as the prototyping board
   	========================================================================= */

// MCC generated application headers 
#include "mcc_generated_files/mcc.h"
#include "lcd.h"
// Application header including global variables and function prototypes
#include "main.h" //

/* =============================================================================
		Main function 
   ========================================================================== */
void main(void){    // Let's make main() the simplest possible.
    init_system();
    
    while(1) {		/* loop forever reading inputs and calculating outputs*/
		read_inputs();
		truth_table();
		write_outputs();
    }		
}	

/* =============================================================================
 Scan inputs and process the bits so that we can set our convenient variables 
 many of the variables are "char" because we can easily monitor them using
 the watch window for debugging purposes.
  =========================================================================== */
void read_inputs (void){     
// Read the port, mask the bit of interest and set the variable 
    Var_SW_S2 = (PORTA & 0b00100000 )>>5; 
       
    Var_SW_S1 = (PORTB & 0b00000001) ;
    
    if ( (Var_SW_S1 == Var_SW_S1_old) && (Var_SW_S2 == Var_SW_S2_old) )
        Var_LCD_Flag = 0;
    else 
        Var_LCD_Flag = 1;
    Var_SW_S1_old = Var_SW_S1; 
    Var_SW_S2_old = Var_SW_S2; 
}

/* =============================================================================
	Driving the output pins. Let's drive the output ports, converting 
	microcontroller variables into electrical signals.    
   ========================================================================== */
void write_outputs(void){
	uint8_t Var_buf; 
    //To write a given output pin:
    // Read and mask to preserve all the bits not to be written
    Var_buf = PORTB & 0b11110011; 
    // Shift the bit of interest to the pin position, compose the port bits (OR)
    // and write the port
    LATB = Var_buf | (Var_LED8 << 3) | (Var_LED7 << 2) ;

}

/* =============================================================================
	Truth table: simple buffers from inputs to outputs 
   ========================================================================== */
void truth_table (void){
        Var_LED8 = Var_SW_S2;
        Var_LED7 = Var_SW_S1; 
        
    if (Var_LCD_Flag == 1){
        LCD_GoTo(0,0);
        if (Var_SW_S1 == 1) LCD_WriteString(" Switch 1 = 0N  ");
        else LCD_WriteString(" Switch 1 = 0FF ");
              
        LCD_GoTo(1,0);
        if (Var_SW_S2 == 1) LCD_WriteString(" Switch 2 = 0N  ");
        else LCD_WriteString(" Switch 2 = 0FF ");
        Var_LCD_Flag = 0; 
    }    
        
}

/* =============================================================================
	Init system: Initialise the microcontroller system 
   ========================================================================== */
void init_system (void){
    
    SYSTEM_Initialize();  //Configuration from MCC
    LCD_Initialize();     // Default LCD on Explorer 8
    

    // Initial message to check that hardware and software is ok
    LCD_GoTo(0,0);
    LCD_WriteString("    CSD         ");
    LCD_GoTo(1,0);
    LCD_WriteString("  Explorer 8    ");
    
}